import requests
import os
import logging
from config import *

SUCCESS_CODE = 200  # Код, возвращяемый при успешном запросе

STATUS_AWAITING_CONFIRMATION = 'wait_confirmation'
STATUS_READY = 'ready'
STATUS_FINISHED = 'done'


"""
Запрашивает информацию всех заказов.
Параметры:

Вовращаемое значение:
    список всех заказов.
"""
def get_orders() -> list:
    orders_url = f'{BASE_URL}{ORDERS_ENDPOINT}'
    response = requests.get(orders_url, headers=HEADERS)

    # Запрос прошел успешно
    if response.status_code == SUCCESS_CODE:
        return response.json()

    # TODO: Вот здесь в идеале должен быть логгер, который выгидывает ошибку / предупреждение.
    print(f'Ошибка при получении заказов: {response.status_code}.')
    return None


"""
Запрос на загрузку файла.

Параметры:
    file_url    - URL до скачиваемого файла на сервере, 
    local_path  - путь до директории, в которую будут скачиваться файлы, 
    headers     - Dictionary of HTTP Headers to send with the Request; 

Возвращаемое значение:
    True    - файлы были скачаны успешно, 
    False   - произошла ошибка при скачивании файлов.
"""
def download_file(file_url : str, 
                  local_path : str, 
                  headers : dict) -> bool:
    try:
        # Выполнение запроса на загрузку файла
        response = requests.get(file_url, headers=headers)
        response.raise_for_status()  # Проверка на наличие ошибок HTTP

        # Проверка, что папки существуют, и создание их при необходимости
        os.makedirs(os.path.dirname(local_path), exist_ok=True)
        
        # Сохранение файла
        with open(local_path, 'wb') as f:
            f.write(response.content)

        print(f'Файл {file_url} успешно загружен в {local_path}.')
        return True

    except requests.exceptions.HTTPError as http_err:
        print(f'Ошибка HTTP: {http_err}.')
        return False

    except Exception as err:
        print(f'Ошибка: {err}.')
        return False


"""
Устанавливает статус заказа.

Параметры:
    order_id - ID заказа, 
    status   - новый статус заказа; 

Возвращаемое значение:
    изменение статуса заказа прошло успешно / произошла ошибка при попытке изменения статуса заказа, 
    код ответа сервера.
"""
def set_order_status(order_id : int, 
                     status : str = None):
    url = f'{BASE_URL}api/mark-order-processed/{order_id}/'
    data = {'status': status}
    response = requests.post(url, headers=HEADERS, json=data)
    code = response.status_code

    # Изменение статуса прошло успешно
    if code == SUCCESS_CODE:
        print(f'Заказ {order_id} успешно помечен как выполненный.')
        return True, code

    # Произошла ошибка при попытке изменения статуса заказа
    print(f'Ошибка при обновлении статуса заказа {order_id}: {code}')
    return False, code


"""
Помечает заказ, как выполненный.

Параметры:
    order_id - ID заказа;

Возвращаемое значение:
    изменение статуса заказа прошло успешно / произошла ошибка при попытке изменения статуса заказа, 
    код ответа сервера.
"""
def mark_order_as_processed(order_id):
    return set_order_status(order_id=order_id, status=STATUS_FINISHED)
